/* This test driver program encodes a data record and writes the encoded 
   result to an output file.. */

#include <stdio.h>
#include <stdlib.h>
#include "RNSAP-Constants.h"
#include "RNSAP-Containers.h"
#include "RNSAP-IEs.h"
#include "RNSAP-PDU-Contents.h"
#include "RNSAP-PDU-Descriptions.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxFile.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   CommonMeasurementInitiationRequest comMeasInitReq;
   InitiatingMessage initMsg;
   RNSAP_PDU     pduData;
   const OSOCTET* msgptr;
   OSOCTET      msgbuf[MAXMSGLEN];
   OSBOOL       verbose = FALSE, trace = TRUE, aligned = TRUE;
   OSCTXT       ctxt;
   const char*  filename = "message.dat";
   int		i, len, stat;

   DECLARE_MEMLEAK_DETECTOR;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   if (rtInitContext (&ctxt) != 0) {
      rtxErrPrint (&ctxt);
      return -1;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, msgbuf, sizeof(msgbuf), aligned);
   pu_setTrace (&ctxt, trace);

   /* Populate CommonMeasurementInitiationRequest structure */

   asn1Init_CommonMeasurementInitiationRequest (&comMeasInitReq);

   stat = asn1Append_CommonMeasurementInitiationRequest_IEs_id_MeasurementID
      (&ctxt, &comMeasInitReq.protocolIEs, 555);
   if (0 != stat) return stat;

   {
      CommonMeasurementObjectType_CM_Rqst* pvalue = 
         rtxMemAllocType (&ctxt, CommonMeasurementObjectType_CM_Rqst);
      if (0 == pvalue) return RTERR_NOMEM;

      pvalue->t = T_CommonMeasurementObjectType_CM_Rqst_cell;
      pvalue->u.cell = rtxMemAllocType (&ctxt, Cell_CM_Rqst);
      if (0 == pvalue->u.cell) {
         rtxMemFreePtr (&ctxt, pvalue);
         return RTERR_NOMEM;
      }
      asn1Init_Cell_CM_Rqst (pvalue->u.cell);
      pvalue->u.cell->uC_ID.rNC_ID = 1234;
      pvalue->u.cell->uC_ID.c_ID = 5678;
      pvalue->u.cell->m.timeSlotPresent = 1;
      pvalue->u.cell->timeSlot = 10;
      
      stat = asn1Append_CommonMeasurementInitiationRequest_IEs_id_CommonMeasurementObjectType_CM_Rqst
         (&ctxt, &comMeasInitReq.protocolIEs, pvalue);
      if (0 != stat) return stat;
   }

   stat = asn1Append_CommonMeasurementInitiationRequest_IEs_id_CommonMeasurementType
      (&ctxt, &comMeasInitReq.protocolIEs, CommonMeasurementType_load);
   if (0 != stat) return stat;

   /* Add extension elements */

   comMeasInitReq.m.protocolExtensionsPresent = 1;

   asn1Append_CommonMeasurementInitiationRequest_Extensions_id_MeasurementRecoveryBehavior (&ctxt, &comMeasInitReq.protocolExtensions);

   asn1Append_CommonMeasurementInitiationRequest_Extensions_id_GANSS_Time_ID (&ctxt, &comMeasInitReq.protocolExtensions, 5);

   /* Populate InitiatingMessage structure */

   asn1Init_InitiatingMessage (&initMsg);
   initMsg.procedureID.procedureCode = ASN1V_id_commonMeasurementInitiation;
   initMsg.procedureID.ddMode = ProcedureID_ddMode_common;
   initMsg.criticality = Criticality_reject;
   initMsg.transactionID.t = T_TransactionID_shortTransActionId;
   initMsg.transactionID.u.shortTransActionId = 22;
   initMsg.value.t = T_RNSAPPDUDescr_RNSAP_EP_commonMeasurementInitiation;
   initMsg.value.u.commonMeasurementInitiation = &comMeasInitReq;

   /* Populate RNSAP-PDU structure */

   pduData.t = T_RNSAP_PDU_initiatingMessage;
   pduData.u.initiatingMessage = &initMsg;

   /* Encode the entire message */

   if ((stat = asn1PE_RNSAP_PDU (&ctxt, &pduData)) == 0) {
      if (trace) {
         printf ("\n******\n");
         printf ("RNSAP PDU encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "userInfo");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }

   msgptr = pe_GetMsgPtr (&ctxt, &len);

   /* Write the encoded message out to the output file */

   stat = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != stat) {
      rtxErrPrint (&ctxt);
      return -1;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
